/*-
 * Copyright (c) 1999,2000 Qube Software, Ltd.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1. Redistributions of the source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Any redistribution solely in binary form must conspicuously
 *    reproduce the following disclaimer in documentation provided with the
 *    binary redistribution.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
 * OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
 * NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
 * SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
 * 
 * $Id: XtQ.c,v 1.4 2000/06/21 19:03:01 n_hibma Exp $
 */

/* Derived from XConsortium: Command.c,v 1.79 94/04/17 20:11:58 kaleb Exp */

/***********************************************************

Copyright (c) 1987, 1988, 1994  X Consortium

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the X Consortium shall not be
used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the X Consortium.


Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the name of Digital not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

/*
 * XtQ.c - Q Xt based Widget
 */

#ifdef DEBUG
#include <stdio.h>
#define DPF(x)		printf x
#else
#define DPF(x)		/* nop */
#endif

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "XtQP.h"

/****************************************************************
 *
 * Full class record constant
 *
 ****************************************************************/

/* Private Data */

static char defaultTranslations[] =
"<EnterWindow>:	 mousefocus()\n\
<LeaveWindow>:	 mousefocus()\n\
<ButtonPress>:	 mousebutton()\n\
<Motion>:	 mousemove()\n\
<ButtonRelease>: mousebutton()\n\
<FocusIn>:	 keyfocus()\n\
<FocusOut>:	 keyfocus()\n\
<KeyPress>:	 key()\n\
<KeyRelease>:	 key()";

static XtResource resources[] = { 
   {XtNcallback, XtCCallback, XtRCallback, sizeof(XtPointer), 
      XtOffsetOf(QRec, q.callbacks), XtRCallback, (XtPointer)NULL},
};

static void Resize(Widget widget);
static void MouseFocus(Widget widget, XEvent *event, String *params,
			Cardinal *num_params);
static void KeyFocus(Widget widget, XEvent *event, String *params,
			Cardinal *num_params);
static void MouseButton(Widget widget, XEvent *event, String *params,
			Cardinal *num_params);
static void MouseMove(Widget widget, XEvent *event, String *params,
			Cardinal *num_params);
static void Key(Widget widget, XEvent *event, String *params,
			Cardinal *num_params);


static XtActionsRec actionsList[] = {
	{"mousefocus",	MouseFocus},
	{"keyfocus",	KeyFocus},
	{"mousebutton",	MouseButton},
	{"mousemove",	MouseMove},
	{"key",		Key}
};

QClassRec qClassRec = {
  {
    (WidgetClass) &widgetClassRec,	/* superclass		  */	
    "Q",				/* class_name		  */
    sizeof(QRec),			/* size			  */
    NULL,				/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    NULL,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    actionsList,			/* actions		  */
    XtNumber(actionsList),		/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    TRUE,				/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    Resize,				/* resize		  */
    NULL,				/* expose		  */
    NULL,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    defaultTranslations,		/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    0,                                     /* field not used    */
  },  /* QClass fields initialization */
};

  /* for public consumption */
WidgetClass qWidgetClass = (WidgetClass) &qClassRec;

/***************************
*
*  Action Procedures
*
***************************/

static void
Resize(Widget widget)
{
    XtCallCallbackList(widget, ((QWidget)widget)->q.callbacks, (XtPointer)NULL);
}

static void
MouseFocus(Widget widget, XEvent *xevent, String *params, Cardinal *num_params)
{
    int focus = (xevent->type == EnterNotify? 1:0);

    /* ... */
}

static void
KeyFocus(Widget widget, XEvent *xevent, String *params, Cardinal *num_params)
{
    int focus = (xevent->type == FocusIn? 1:0);

    /* ... */

    DPF(("Key Focus (%d) %sset in 0x%08x\n", xevent->type, focus? "":"un",
         XtWindow(widget)));
}

static void
MouseButton(Widget widget, XEvent *xevent, String *params, Cardinal *num_params)
{
    int pressed = (xevent->type == ButtonPress? 1:0);
    int buttons = 1 << xevent->xbutton.button;

    /* ... */
    DPF(("Mouse button 0x%08x %s\n", buttons, (pressed? "pressed":"released")));
}

static void
MouseMove(Widget widget, XEvent *xevent, String *params, Cardinal *num_params)
{
    int dx = xevent->xmotion.x;
    int dy = xevent->xmotion.y;

    /* ... */
}

static void
Key(Widget widget, XEvent *xevent, String *params, Cardinal *num_params)
{
    int pressed = (xevent->type == KeyPress? 1:0);
    int key = XKeycodeToKeysym(XtDisplay(widget), xevent->xkey.keycode, 0);

    /* ... */
    DPF(("Key 0x%04x %s (state = 0x%08x)\n",
         key, pressed? "pressed":"released", xevent->xkey.state));
}

